from django.db.models import BooleanField
from django.db.models import Case
from django.db.models import Value
from django.db.models import When
from m3_ext.ui import all_components as ext
from m3_ext.ui.misc import ExtDataStore

from kinder.core.direct.models import DRS

from .models import ApplicantAnswer
from .models import ApplicantAnswersEnum


APPLICANT_ANSWER_PARAM = 'applicant_answer'


def add_applicant_answer_column(columns, ext_result):
    """Добавляет столбец "Ответ заявителя" в реестр направлений."""

    _columns = columns.copy()
    index = [
        _columns.index(x) for x in _columns if x[0] == 'group.unit.name'
    ][0]

    _columns.insert(
        index + 1, (APPLICANT_ANSWER_PARAM, 'Ответ заявителя', 50)
    )

    return _columns


def add_applicant_answer_column_filter(win, name, ext_result):
    """
    Добавляет колоночный фильтр для столбца "Ответ заявителя"
    в реестре направлений.
    """

    applicant_answer_filter = ext.ExtDictSelectField(
        region='north',
        anchor='100%',
        ask_before_deleting=False,
        allow_blank=True,
        hide_edit_trigger=True,
        hide_clear_trigger=False,
        hide_dict_select_trigger=True,
        hide_trigger=False,
    )

    filter_store = list(ApplicantAnswersEnum.values_for_filter.items())
    applicant_answer_filter.set_store(ExtDataStore(filter_store))

    win.add_grid_column_filter(
        APPLICANT_ANSWER_PARAM, applicant_answer_filter.render())


def add_applicant_answer_data(obj, ext_result):
    """
    Добавляет значение для атрибута столбца "Ответ заявителя" и атрибут для
    окрашивания строк в синий цвет, если "Ответ заявителя" = True и направление
    в статусе "Направлен в ДОО".

    """

    if obj.applicant_answer is None:
        obj.applicant_answer = ''
    else:
        # Атрибут для окрашивания строки в синий цвет
        obj.applicant_answer_highlight = obj.status.code == DRS.REGISTER
        # Человекочитаемое значение поля "Ответ заявителя"
        obj.applicant_answer = ApplicantAnswersEnum.values.get(
            obj.applicant_answer, '')

    return obj


def applicant_answer_update_filter_list(fields_list, ext_result):
    """Обновляет список стоблцов с кастомными колоночными фильтрами."""

    return fields_list.append(APPLICANT_ANSWER_PARAM)


def apply_applicant_answer_filter(query, context, ext_result):
    """
    Фильтрация направлений через колоночный фильтр и сортировка по стоблцу
    "Ответ заявителя".

    """

    query = query.annotate(
        applicant_answer=Case(
            When(smev3_v321_applicant_answer__isnull=True, then=Value(None)),
            When(smev3_v321_applicant_answer__answer=True, then=Value(True)),
            When(smev3_v321_applicant_answer__answer=False, then=Value(False)),
            output_field=BooleanField()
        )
    )

    # Фильтрация запроса
    if hasattr(context, APPLICANT_ANSWER_PARAM):
        param = getattr(context, APPLICANT_ANSWER_PARAM)
        param = True if param == 'true' else False if param == 'false' else None
        query = query.filter(applicant_answer=param)

    return query


def apply_need_confirmation_only_filter(query, context, ext_result):
    """
    Фильтрация направлений в зависимости от значения в чекбоксе
    "На подтверждение"
    """
    if int(getattr(context, 'need_confirmation_only', 0)):
        query = query.filter(
            status__code=DRS.REGISTER,
            smev3_v321_applicant_answer__isnull=False)

    return query
