from django.db.models import Exists
from django.db.models import OuterRef

from aio_client.provider.api import PostProviderRequest
from aio_client.provider.api import push_request
from kinder.core.declaration.models import Declaration
from kinder.core.declaration_status.enum import DSS
from kinder.core.queue_module.helpers import get_position_info
from kinder.core.queue_module.models import DeclarationPositionLog

from concentrator.smev3_v321.base import constants as base_enums
from concentrator.smev3_v321.base.utils import BaseExecutor
from concentrator.smev3_v321.base.utils import SMEV3Response
from concentrator.smev3_v321.base.utils import render_type2xml
from concentrator.smev3_v321.service_types import kinder_conc

from .constants import GetApplicationQueueReasonRequest
from ..helpers import get_declaration_by_client_or_portal_id


class GetApplicationQueueReasonRequestExecutor(BaseExecutor):
    """Исполнитель сервиса GetApplicationQueueReasonRequest."""

    name_service = 'GetApplicationQueueReasonRequest'
    type_service = kinder_conc.GetApplicationQueueReasonRequestType

    @classmethod
    def prepare_query(cls, request_body):
        """Выполняет подготовку запроса."""

        # Дата с и Дата по, полученные из запроса.
        period_start, period_end = (
            request_body.GetApplicationQueueReasonRequest.PeriodStart,
            request_body.GetApplicationQueueReasonRequest.PeriodEnd)

        # Подзапрос на выборку существования лога изменения позиции
        # заявления в очереди по организации МО за полученный период.
        subquery_position_log = Exists(
            DeclarationPositionLog.objects.filter(
                created__date__range=(period_start, period_end),
                declaration=OuterRef('id'), unit=OuterRef('mo')))

        query = (
            Declaration.objects.select_related('status').annotate(
                position_log_exists=subquery_position_log
            ))

        return query

    @classmethod
    def validate(cls, declaration):
        """Выполняет валидацию данных заявления.

        :param declaration: Заявление
        :type declaration: Optional[Declaration]

        :return: Словарь с данным для ответа (changeOrderInfo)
        :rtype: dict
         """

        # Проверка существования заявления.
        if not declaration:
            return {
                'statusCode': kinder_conc.statusCodeType(
                    GetApplicationQueueReasonRequest.values.get(
                        GetApplicationQueueReasonRequest.NOT_EXISTS)),
                'comment': GetApplicationQueueReasonRequest.NOT_EXISTS
            }

        # Заявка должна быть со статусом, участвующим в очереди.
        if declaration.status.code not in DSS.status_queue_full():
            return {
                'statusCode': kinder_conc.statusCodeType(
                    GetApplicationQueueReasonRequest.values.get(
                        GetApplicationQueueReasonRequest.NOT_QUEUED)),
                'comment': GetApplicationQueueReasonRequest.NOT_QUEUED
            }

        # Проверка изменения позиции заявления в очереди.
        if not declaration.position_log_exists:
            return {
                'statusCode': kinder_conc.statusCodeType(
                    GetApplicationQueueReasonRequest.values.get(
                        GetApplicationQueueReasonRequest.NO_CHANGES)),
                'comment': GetApplicationQueueReasonRequest.NO_CHANGES
            }

        return {}

    @classmethod
    def process(cls, message, request_body):

        declaration, _ = get_declaration_by_client_or_portal_id(
            cls.prepare_query(request_body),
            request_body.GetApplicationQueueReasonRequest.orderId)

        response_body = render_type2xml(
            cls.get_response(request_body, declaration),
            name_type='FormDataResponse')

        response = push_request(PostProviderRequest(
            origin_message_id=message['origin_message_id'],
            body=response_body,
            message_type=message['message_type'],
            replay_to=message['replay_to'],
            is_test_message=message['is_test_message']))

        return SMEV3Response(
            response,
            {
                'method_name': cls.name_service,
                'response': response_body,
                'result': base_enums.SUCCESS_MESSAGE
            })

    @classmethod
    def get_response(cls, request_body, declaration):
        """Формирует ответ на GetApplicationQueueReasonRequest.

        :param request_body: Тело запроса
        :param declaration: Заявление
        :type declaration: Declaration
        :return: ответ
        :rtype: FormDataResponseType
        """
        if not declaration.position_log_exists:
            return kinder_conc.FormDataResponseType(
                GetApplicationQueueReasonResponse=(
                    kinder_conc.GetApplicationQueueReasonResponseType(
                        orderId=request_body.GetApplicationQueueReasonRequest.orderId,
                        IncreaseQueue=0, GotAPlace=0,
                        IncreaseBenefits=0)))

        # Дата с и Дата по, полученные из запроса.
        date_range = (
            request_body.GetApplicationQueueReasonRequest.PeriodStart,
            request_body.GetApplicationQueueReasonRequest.PeriodEnd)

        validation_errors = cls.validate(declaration)
        if validation_errors:
            return kinder_conc.FormDataResponseType(
                changeOrderInfo=kinder_conc.changeOrderInfoType(**{
                    'orderId': kinder_conc.orderIdType(
                        request_body.GetApplicationQueueReasonRequest.orderId),
                    **validation_errors
                }))

        unit = declaration.mo
        unit_kind_id = unit.kind_id

        increase_queue, got_place, increase_benefits = get_position_info(
            date_range, declaration.id, unit_kind_id, unit.id)

        response = kinder_conc.FormDataResponseType(
            GetApplicationQueueReasonResponse=(
                kinder_conc.GetApplicationQueueReasonResponseType(
                    orderId=kinder_conc.orderIdType(
                        request_body.GetApplicationQueueReasonRequest.orderId),
                    IncreaseQueue=increase_queue, GotAPlace=got_place,
                    IncreaseBenefits=increase_benefits)))

        return response
